# -*- coding: utf-8 -*-
# cython: language_level=3
# Copyright (c) 2022 Crisp Crow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Exceptions that can be thrown by this library."""

from __future__ import annotations

__all__: typing.Sequence[str] = ('OnewotError', 'EntityNotFound', 'UserNotFound')

import typing

import attr


@attr.define(slots=True, frozen=True, auto_exc=True)
class OnewotError(RuntimeError):
    """Base for an error raised by this API."""

    code: str = attr.field()
    message: str = attr.field()
    field: str = attr.field()
    value: typing.Union[str, int] = attr.field()


@attr.define(auto_exc=True)
class EntityNotFound(OnewotError):
    """Exception thrown when searched entity has invalid ID or name."""

    field: str = attr.field(init=False)


@attr.define(slots=True, frozen=True, auto_exc=True)
class UserNotFound(EntityNotFound):
    """Exception thrown when searched user has invalid ID or name."""
