# -*- coding: utf-8 -*-
# cython: language_level=3
# Copyright (c) 2022 Crisp Crow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Package metadata."""

import typing


__name__: typing.Final[str] = 'onewot'
__author__: typing.Final[str] = 'Crisp Crow'
__description__: typing.Final[str] = 'The WotBlitz API wrapper for Python3'
__copyright__: typing.Final[str] = '© 2022-present Crisp Crow'
__email__: typing.Final[str] = 'minikslyh@gmail.com'
__license__: typing.Final[str] = 'MIT'
__url__: typing.Final[str] = 'https://github.com/CrispCrow/Onewot'
__issue_tracker__: typing.Final[str] = 'https://github.com/CrispCrow/Onewot/issues'
__version__: typing.Final[str] = '0.3'
